@@ -2,7 +2,7 @@
  Project: Graphos
  GRBezierPath.m
 
- Copyright (C) 2000-2011 GNUstep Application Project
+ Copyright (C) 2000-2013 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -31,76 +31,17 @@ static double k = 0.025;
 
 @implementation GRBezierPath
 
-- (id)initInView:(GRDocView *)aView zoomFactor:(float)zf
-{
-    self = [super init];
-    if(self)
-    {
-        docView = aView;
-        zmFactor = zf;
-        myPath = [[NSBezierPath bezierPath] retain];
-        [myPath setCachesBezierPath: NO];
-        controlPoints = [[NSMutableArray alloc] initWithCapacity: 1];
-        flatness = 0.0;
-        miterlimit = 2.0;
-        linewidth = 1.5;
-        linejoin = 0;
-        linecap = 0;
-        stroked = YES;
-        filled = NO;
-        visible = YES;
-        locked = NO;
-        strokeColor = [[NSColor blackColor] retain];
-        fillColor = [[NSColor whiteColor] retain];
-        editor = [[GRBezierPathEditor alloc] initEditor:self];
-    }
-    return self;
-}
 
 /** initializes by using the properties array as defaults */
 - (id)initInView:(GRDocView *)aView
-      zoomFactor:(float)zf
+      zoomFactor:(CGFloat)zf
       withProperties:(NSDictionary *)properties
 {
-  self = [self initInView:aView zoomFactor:zf];
+  self = [super initInView:aView zoomFactor:zf withProperties:properties];
   if(self)
     {
-      NSColor *newColor;
-      NSString *val;
-
-      val = [properties objectForKey: @"flatness"];
-      if (val != nil)
-	[self setFlat: [val floatValue]];
-
-      val = [properties objectForKey: @"linejoin"];
-      if (val != nil)
-	[self setLineJoin: [val intValue]];
-
-      val = [properties objectForKey: @"linecap"];
-      if (val != nil)
-	[self setLineCap: [val intValue]];
-
-      val = [properties objectForKey: @"miterlimit"];
-      if (val != nil)
-	[self setMiterLimit: [val floatValue]];
-
-      val = [properties objectForKey: @"linewidth"];
-      if (val != nil)
-      [self setLineWidth: [val floatValue]];
-
-      val = [properties objectForKey: @"stroked"];
-      if (val != nil)
-	[self setStroked: (BOOL)[val intValue]];
-      newColor = (NSColor *)[properties objectForKey: @"strokecolor"];
-      if (newColor != nil)
-	[self setStrokeColor: newColor];
-
-      val = [properties objectForKey: @"filled"];
-      if (val != nil)
-	[self setFilled: (BOOL)[val intValue]];
-      newColor = (NSColor *)[properties objectForKey: @"fillcolor"];
-      if (newColor != nil)
-	[self setFillColor: newColor];
+      controlPoints = [[NSMutableArray alloc] initWithCapacity: 1];
+      editor = [[GRBezierPathEditor alloc] initEditor:self];
     }
 
   return self;
@@ -108,7 +49,7 @@ static double k = 0.025;
 
 - (id)initFromData:(NSDictionary *)description
         inView:(GRDocView *)aView
-        zoomFactor:(float)zf
+        zoomFactor:(CGFloat)zf
 {
     NSArray *psops, *linearr;
     NSString *str;
@@ -124,6 +65,7 @@ static double k = 0.025;
 	float fillCol[4];
 	float strokeAlpha;
 	float fillAlpha;
+	id obj;
 
         editor = [[GRBezierPathEditor alloc] initEditor:self];
         docView = aView;
@@ -187,41 +129,76 @@ static double k = 0.025;
         linecap = [[description objectForKey: @"linecap"] intValue];
         miterlimit = [[description objectForKey: @"miterlimit"] floatValue];
         linewidth = [[description objectForKey: @"linewidth"] floatValue];
-
-        stroked = (BOOL)[[description objectForKey: @"stroked"] intValue];
+	obj = [description objectForKey: @"stroked"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        stroked = [obj boolValue];
+	strokeAlpha = [[description objectForKey: @"strokealpha"] floatValue];
         str = [description objectForKey: @"strokecolor"];
         linearr = [str componentsSeparatedByString: @" "];
-
-        strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
-        strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
-        strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
-        strokeCol[3] = [[linearr objectAtIndex: 3] floatValue];
-        strokeAlpha = [[description objectForKey: @"strokealpha"] floatValue];
-	strokeColor = [NSColor colorWithDeviceCyan: strokeCol[0]
-					   magenta: strokeCol[1]
-					    yellow: strokeCol[2]
-					     black: strokeCol[3]
-					     alpha: strokeAlpha];
-	strokeColor = [[strokeColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
-
-        filled = (BOOL)[[description objectForKey: @"filled"] intValue];
+	if ([linearr count] == 3)
+	  {
+	    strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    strokeColor = [NSColor colorWithCalibratedRed: strokeCol[0]
+				   green: strokeCol[1]
+				   blue: strokeCol[2]
+				   alpha: strokeAlpha];
+	    [strokeColor retain];
+	  }
+	else
+	  {
+	    strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    strokeCol[3] = [[linearr objectAtIndex: 3] floatValue];
+	    strokeColor = [NSColor colorWithDeviceCyan: strokeCol[0]
+				   magenta: strokeCol[1]
+				   yellow: strokeCol[2]
+				   black: strokeCol[3]
+				   alpha: strokeAlpha];
+	    strokeColor = [[strokeColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+	  }
+        obj = [description objectForKey: @"filled"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        filled = [obj boolValue];
+        fillAlpha = [[description objectForKey: @"fillalpha"] floatValue];
         str = [description objectForKey: @"fillcolor"];
         linearr = [str componentsSeparatedByString: @" "];
-        fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
-        fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
-        fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
-        fillCol[3] = [[linearr objectAtIndex: 3] floatValue];
-        fillAlpha = [[description objectForKey: @"fillalpha"] floatValue];
-	fillColor = [NSColor colorWithDeviceCyan: fillCol[0]
-					 magenta: fillCol[1]
-					  yellow: fillCol[2]
-					   black: fillCol[3]
-					   alpha: fillAlpha];
-	fillColor = [[fillColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
-
-
-        visible = (BOOL)[[description objectForKey: @"visible"] intValue];
-        locked = (BOOL)[[description objectForKey: @"locked"] intValue];
+	if ([linearr count] == 3)
+	  {
+	    fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    fillColor = [NSColor colorWithCalibratedRed: fillCol[0]
+				 green: fillCol[1]
+				 blue: fillCol[2]
+				 alpha: fillAlpha];
+	    [fillColor retain];
+	  }
+	else
+	  {
+	    fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    fillCol[3] = [[linearr objectAtIndex: 3] floatValue];
+	    fillColor = [NSColor colorWithDeviceCyan: fillCol[0]
+				 magenta: fillCol[1]
+				 yellow: fillCol[2]
+				 black: fillCol[3]
+				 alpha: fillAlpha];
+	    fillColor = [[fillColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+	  }
+        obj = [description objectForKey: @"visible"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        visible = [obj boolValue];
+        obj = [description objectForKey: @"locked"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        locked = [obj boolValue];
     }
 
     return self;
@@ -236,26 +213,20 @@ static double k = 0.025;
     NSBezierPathElement type;
     NSPoint p[3];
     int i;
-    NSColor *strokeColorCMYK;
-    NSColor *fillColorCMYK;
-    float strokeCol[4];
-    float fillCol[4];
+    float strokeCol[3];
+    float fillCol[3];
     float strokeAlpha;
     float fillAlpha;
 
-    strokeColorCMYK = [strokeColor colorUsingColorSpaceName: NSDeviceCMYKColorSpace]; 
-    strokeCol[0] = [strokeColorCMYK cyanComponent];
-    strokeCol[1] = [strokeColorCMYK magentaComponent];
-    strokeCol[2] = [strokeColorCMYK yellowComponent];
-    strokeCol[3] = [strokeColorCMYK blackComponent];
-    strokeAlpha = [strokeColorCMYK alphaComponent];
-
-    fillColorCMYK = [fillColor colorUsingColorSpaceName: NSDeviceCMYKColorSpace]; 
-    fillCol[0] = [fillColorCMYK cyanComponent];
-    fillCol[1] = [fillColorCMYK magentaComponent];
-    fillCol[2] = [fillColorCMYK yellowComponent];
-    fillCol[3] = [fillColorCMYK blackComponent];
-    fillAlpha = [fillColorCMYK alphaComponent];
+    strokeCol[0] = [strokeColor redComponent];
+    strokeCol[1] = [strokeColor greenComponent];
+    strokeCol[2] = [strokeColor blueComponent];
+    strokeAlpha = [strokeColor alphaComponent];
+ 
+    fillCol[0] = [fillColor redComponent];
+    fillCol[1] = [fillColor greenComponent];
+    fillCol[2] = [fillColor blueComponent];
+    fillAlpha = [fillColor alphaComponent];
 
     dict = [NSMutableDictionary dictionaryWithCapacity: 1];
     [dict setObject: @"path" forKey: @"type"];
@@ -270,24 +241,20 @@ static double k = 0.025;
     [dict setObject: str forKey: @"miterlimit"];
     str = [NSString stringWithFormat: @"%.3f", linewidth];
     [dict setObject: str forKey: @"linewidth"];
-    str = [NSString stringWithFormat: @"%i", stroked];
-    [dict setObject: str forKey: @"stroked"];
-    str = [NSString stringWithFormat: @"%.3f %.3f %.3f %.3f",
-        strokeCol[0], strokeCol[1], strokeCol[2], strokeCol[3]];
+    [dict setObject: [NSNumber numberWithBool:stroked] forKey: @"stroked"];
+    str = [NSString stringWithFormat: @"%.3f %.3f %.3f",
+        strokeCol[0], strokeCol[1], strokeCol[2]];
     [dict setObject: str forKey: @"strokecolor"];
     str = [NSString stringWithFormat: @"%.3f", strokeAlpha];
     [dict setObject: str forKey: @"strokealpha"];
-    str = [NSString stringWithFormat: @"%i", filled];
-    [dict setObject: str forKey: @"filled"];
-    str = [NSString stringWithFormat: @"%.3f %.3f %.3f %.3f",
-        fillCol[0], fillCol[1], fillCol[2], fillCol[3]];
+    [dict setObject:[NSNumber numberWithBool:filled] forKey: @"filled"];
+    str = [NSString stringWithFormat: @"%.3f %.3f %.3f",
+        fillCol[0], fillCol[1], fillCol[2]];
     [dict setObject: str forKey: @"fillcolor"];
     str = [NSString stringWithFormat: @"%.3f", fillAlpha];
     [dict setObject: str forKey: @"fillalpha"];
-    str = [NSString stringWithFormat: @"%i", visible];
-    [dict setObject: str forKey: @"visible"];
-    str = [NSString stringWithFormat: @"%i", locked];
-    [dict setObject: str forKey: @"locked"];
+    [dict setObject:[NSNumber numberWithBool:visible] forKey: @"visible"];
+    [dict setObject:[NSNumber numberWithBool:locked] forKey: @"locked"];
 
     psops = [NSMutableArray arrayWithCapacity: 1];
     for(i = 0; i < [myPath elementCount]; i++)
@@ -658,12 +625,10 @@ static double k = 0.025;
     [self remakePath];
 }
 
-- (void)setZoomFactor:(float)f
+- (void)setZoomFactor:(CGFloat)f
 {
     int i;
 
-    linewidth = linewidth / zmFactor * f;
-
     zmFactor = f;
     for(i = 0; i < [controlPoints count]; i++)
         [[controlPoints objectAtIndex: i] setZoomFactor: zmFactor];
@@ -671,6 +636,11 @@ static double k = 0.025;
     [self remakePath];
 }
 
+- (BOOL)objectHitForSelection:(NSPoint)p
+{
+  return [self onPathBorder:p];
+}
+
 - (BOOL)onPathBorder:(NSPoint)p
 {
     int i;
@@ -747,13 +717,16 @@ static double k = 0.025;
 
 - (void)draw
 {
-    GRBezierControlPoint *cp;
-    NSRect r;
-    int i;
-    NSBezierPath *bzp;
+  GRBezierControlPoint *cp;
+  NSRect r;
+  int i;
+  NSBezierPath *bzp;
+  float linew;
+    
+  if(![controlPoints count] || !visible)
+    return;
 
-    if(![controlPoints count] || !visible)
-        return;
+  linew =  linewidth * zmFactor;
 
     bzp = [NSBezierPath bezierPath];
     if(filled)
@@ -768,7 +741,7 @@ static double k = 0.025;
       [NSGraphicsContext saveGraphicsState];
       [myPath setLineJoinStyle:linejoin];
       [myPath setLineCapStyle:linecap];
-      [myPath setLineWidth:linewidth];
+      [myPath setLineWidth:linew];
       [strokeColor set];
       [myPath stroke];
       [NSGraphicsContext restoreGraphicsState];
