@@ -2,7 +2,7 @@
  Project: Graphos
  GRBox.h
 
- Copyright (C) 2007-2011 GNUstep Application Project
+ Copyright (C) 2007-2013 GNUstep Application Project
 
  Author: Ing. Riccardo Mottola
 
@@ -39,11 +39,7 @@
     float rotation;
 }
 
-- (id)initInView:(GRDocView *)aView
-      zoomFactor:(float)zf;
-- (id)initInView:(GRDocView *)aView
-      zoomFactor:(float)zf
-      withProperties:(NSDictionary *)properties;
+- (NSPoint) position;
 - (BOOL)onControlPoint:(NSPoint)p;
 - (void)setStartAtPoint:(NSPoint)aPoint;
 - (void)setEndAtPoint:(NSPoint)aPoint;
