@@ -2,7 +2,7 @@
  Project: Graphos
  GRDocView.h
 
- Copyright (C) 2000-2011 GNUstep Application Project
+ Copyright (C) 2000-2013 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -27,7 +27,6 @@
 
 #import "GRBezierPathEditor.h"
 
-
 #ifndef ASSIGN
 #define ASSIGN(object,value)     ({\
     id __value = (id)(value); \
@@ -56,8 +55,9 @@
 
     NSRect pageRect, a4Rect, zmdRect;
     int zIndex;
-    float zFactor;
+    CGFloat zFactor;
     BOOL isDrawingForPrinting;
+    NSCursor *cur;
 }
 
 - (id)initWithFrame:(NSRect)aRect;
@@ -116,7 +116,13 @@
 - (IBAction)moveSelectedObjectsToBack:(id)sender;
 - (void)unselectOtherObjects:(GRDrawableObject *)anObject;
 - (void)zoomOnPoint:(NSPoint)p zoomOut:(BOOL)isout;
+- (void)zoomOnPoint:(NSPoint)p withFactor:(int)index;
 - (void)movePageFromHandPoint:(NSPoint)handpos;
+- (IBAction)zoom50:(id)sender;
+- (IBAction)zoom100:(id)sender;
+- (IBAction)zoom200:(id)sender;
+- (IBAction)zoomFitPage:(id)sender;
+- (IBAction)zoomFitWidth:(id)sender;
 
 - (IBAction)cut:(id)sender;
 - (IBAction)copy:(id)sender;
