@@ -2,7 +2,7 @@
  Project: Graphos
  GRDocView.m
 
- Copyright (C) 2000-2012 GNUstep Application Project
+ Copyright (C) 2000-2013 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -36,47 +36,46 @@
 
 #define UNDO_ACTION_OBJPROPS @"Change Object Properties"
 
-float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
+#define ZOOM_FACTORS 8
+float zFactors[ZOOM_FACTORS] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6};
 
 @implementation GRDocView
 
 
 - (id)initWithFrame:(NSRect)aRect
 {
-    NSLog (@"initing doc view with win");
-    pageRect = NSMakeRect(0, 0, 695, 942);
-    a4Rect = NSMakeRect(50, 50, 595, 842);
-    zmdRect = NSMakeRect(50, 50, 595, 842);
-
-    self = [super initWithFrame: pageRect];
-    if(self)
+  pageRect = NSMakeRect(0, 0, 695, 942);
+  a4Rect = NSMakeRect(50, 50, 595, 842);
+  zmdRect = NSMakeRect(50, 50, 595, 842);
+  
+  self = [super initWithFrame: pageRect];
+  if(self)
     {
-        NSImage *img;
-        NSCursor *cur;
+      NSImage *img;
         
-        img = [NSImage imageNamed: @"blackarrow.tiff"];
-        cur = [[NSCursor alloc] initWithImage: img hotSpot: NSMakePoint(0, 0)];
-        [cur setOnMouseEntered: YES];
-        [cur setOnMouseExited: YES];
-        [self addTrackingRect: [self frame]
-                        owner: cur userData: NULL
-                 assumeInside: YES];
+      img = [NSImage imageNamed: @"blackarrow.tiff"];
+      cur = [[NSCursor alloc] initWithImage: img hotSpot: NSMakePoint(0, 0)];
+      [cur setOnMouseEntered: YES];
+      [cur setOnMouseExited: YES];
+      [self addTrackingRect: [self frame]
+	    owner: cur userData: NULL
+	    assumeInside: YES];
 
-
-        objects = [[NSMutableArray alloc] initWithCapacity: 1];
-        delObjects = [[NSMutableArray alloc] initWithCapacity: 1];
-        lastObjects = nil;
-        shiftclick = NO;
-        altclick = NO;
-        ctrlclick = NO;
-        zIndex = 2;
-        zFactor = zFactors[zIndex];
+      objects = [[NSMutableArray alloc] initWithCapacity: 1];
+      delObjects = [[NSMutableArray alloc] initWithCapacity: 1];
+      lastObjects = nil;
+      shiftclick = NO;
+      altclick = NO;
+      ctrlclick = NO;
+      zIndex = 2;
+      zFactor = zFactors[zIndex];
     }
-    return self;
+  return self;
 }
 
 - (void)dealloc
 {
+    [cur release];
     [objects release];
     [delObjects release];
     [lastObjects release];
@@ -89,11 +88,11 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
   NSMutableArray *objectOrder;
   NSString *str = nil;
   id obj;
-  int i;
-  int p = 0;
-  int c = 0;
-  int t = 0;
-  int b = 0;
+  NSUInteger i;
+  NSUInteger p = 0;
+  NSUInteger c = 0;
+  NSUInteger t = 0;
+  NSUInteger b = 0;
   
   objsdict = [NSMutableDictionary dictionaryWithCapacity: 1];
   objectOrder = [NSMutableArray arrayWithCapacity: [objects count]];
@@ -136,14 +135,14 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (NSArray *)usedFonts
 {
-    NSMutableArray *usedfonts;
-    NSString *fname;
-    id obj;
-    int i, j;
-    BOOL exist;
+  NSMutableArray *usedfonts;
+  NSString *fname;
+  id obj;
+  NSUInteger i, j;
+  BOOL exist;
 
-    usedfonts = [NSMutableArray arrayWithCapacity: 1];
-    for(i = 0; i < [objects count]; i++)
+  usedfonts = [NSMutableArray arrayWithCapacity: 1];
+  for(i = 0; i < [objects count]; i++)
     {
         exist = NO;
         obj = [objects objectAtIndex: i];
@@ -334,7 +333,8 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 			  zoomFactor: zFactor
 		      withProperties: [objInspector properties]
 			  openEditor: YES];
-  [objects addObject: gdtxt];
+  if (gdtxt)
+    [objects addObject: gdtxt];
   [[gdtxt editor] select];
   [gdtxt release];
   [self setNeedsDisplay: YES];
@@ -381,27 +381,27 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (NSArray *)duplicateObjects:(NSArray *)objs andMoveTo:(NSPoint)p
 {
-    id obj, duplObj;
-    NSMutableArray *duplObjs;
-    int i;
+  id obj, duplObj;
+  NSMutableArray *duplObjs;
+  int i;
 
-    duplObjs = [NSMutableArray arrayWithCapacity: 1];
+  duplObjs = [NSMutableArray arrayWithCapacity: 1];
 
-    for(i = 0; i < [objs count]; i++)
+  for(i = 0; i < [objs count]; i++)
     {
-        obj = [objs objectAtIndex: i];
-        duplObj = [obj copy];
-        [[obj editor] unselect];
-        [duplObj selectAsGroup];
-        [duplObj moveAddingCoordsOfPoint: p];
-        [objects addObject: duplObj];
-        [duplObjs addObject: duplObj];
-	[duplObj release];
+      obj = [objs objectAtIndex: i];
+      duplObj = [obj copy];
+      [[obj editor] unselect];
+      [[duplObj editor] selectAsGroup];
+      [duplObj moveAddingCoordsOfPoint: p];
+      [objects addObject: duplObj];
+      [duplObjs addObject: duplObj];
+      [duplObj release];
     }
-    edind = [objects count] -1;
-    [self setNeedsDisplay: YES];
-
-    return duplObjs;
+  edind = [objects count] -1;
+  [self setNeedsDisplay: YES];
+  
+  return duplObjs;
 }
 
 - (void)updatePrintInfo: (NSPrintInfo *)pi;
@@ -560,242 +560,145 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)startBoxAtPoint:(NSPoint)p
 {
-    NSEvent *nextEvent;
-    GRBox *box;
-    id obj;
-    BOOL isneweditor = YES;
-    int i;
-    NSUndoManager *uMgr;
+  NSEvent *nextEvent;
+  GRBox *box;
+  NSUInteger i;
+  NSUndoManager *uMgr;
     
-    uMgr = [self undoManager];
-    /* save the method on the undo stack */
-    [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
-    [uMgr setActionName:@"Create Box"];
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Create Box"];
     
-    [self saveCurrentObjects];
-    
-    for(i = 0; i < [objects count]; i++)
+  [self saveCurrentObjects];
+
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if([obj isKindOfClass: [GRBox class]])
-            if(![[obj editor] isdone])
-                isneweditor = NO;
+      GRObjectEditor *objEdi;
+            
+      objEdi = [[objects objectAtIndex: i] editor];
+      if (![objEdi isSelect])
+        [objEdi unselect];
     }
 
-    if(isneweditor)
-        for(i = 0; i < [objects count]; i++)
-        {
-            GRObjectEditor *objEdi;
-            
-            objEdi = [[objects objectAtIndex: i] editor];
-            if (![objEdi isSelect])
-                [objEdi unselect];
-        }
+  nextEvent = [[self window] nextEventMatchingMask:
+                               NSLeftMouseUpMask | NSLeftMouseDraggedMask];
+  [self verifyModifiersOfEvent: nextEvent];
 
-    nextEvent = [[self window] nextEventMatchingMask:
-        NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-    [self verifyModifiersOfEvent: nextEvent];
-
-    if([nextEvent type] != NSLeftMouseDragged)
+  if([nextEvent type] != NSLeftMouseDragged)
     {
-       NSLog(@"is not left mouse dragged");
-        if(isneweditor)
-        {
-            [self addBox];
-            box = [objects objectAtIndex: edind];
-            [[box editor] selectForEditing];
-            [box setStartAtPoint: p];
-            [self setNeedsDisplay: YES];
-            return;
-        } else
-        {
-            box = [objects objectAtIndex: edind];
-            [[box editor] selectForEditing];
-//            if(shiftclick)
-//                p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
-            [box setEndAtPoint: p];
-            [self setNeedsDisplay: YES];
-            return;
-        }
-    } else
+      NSLog(@"is not left mouse dragged");
+ 
+      [self addBox];
+      box = [objects objectAtIndex: edind];
+      [[box editor] selectForEditing];
+      [box setStartAtPoint: p];
+      [self setNeedsDisplay: YES];
+      return;
+    }
+  else
     {
-       NSLog(@"is left mouse dragged");
-        if(isneweditor)
-        {
-       NSLog(@"is new editor");
-            [self addBox];
-            box = [objects objectAtIndex: edind];
-            [[box editor] selectForEditing];
-            [box setStartAtPoint: p];
-            [self setNeedsDisplay: YES];
-        } else
-        {
-       NSLog(@"is old editor");
-            box = [objects objectAtIndex: edind];
-            [[box editor] selectForEditing];
-//            if(shiftclick)
-//                p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
-            [box setEndAtPoint: p];
-            [self setNeedsDisplay: YES];
-        }
+      NSLog(@"is left mouse dragged");
+ 
+      [self addBox];
+      box = [objects objectAtIndex: edind];
+      [[box editor] selectForEditing];
+      [box setStartAtPoint: p];
+      [box setEndAtPoint: p];
+      [self setNeedsDisplay: YES];
 
-        do
-        {
-            p = [nextEvent locationInWindow];
-            p = [self convertPoint: p fromView: nil];
-//            if(shiftclick)
-//                p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
-
-            [box setEndAtPoint: p];
-
-            [self setNeedsDisplay: YES];
-
-            nextEvent = [[self window] nextEventMatchingMask:
-                        NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-            [self verifyModifiersOfEvent: nextEvent];
-        } while([nextEvent type] != NSLeftMouseUp);
-        if (isneweditor)
-        {
-            [[box editor] unselect];
-            [[box editor] selectAsGroup];
-        }
+      [self moveControlPointOfEditor: (GRBezierPathEditor *)[box editor] toPoint: p];
+ 
+      [[box editor] unselect];
+      [[box editor] selectAsGroup];
     }
 }
 
 /* this has a lot in common with startBoxAtPoint */
 - (void)startCircleAtPoint:(NSPoint)p
 {
-    NSEvent *nextEvent;
-    GRCircle *circle;
-    id obj;
-    BOOL isneweditor = YES;
-    int i;
-    NSUndoManager *uMgr;
+  NSEvent *nextEvent;
+  GRCircle *circle;
+  NSUInteger i;
+  NSUndoManager *uMgr;
     
-    uMgr = [self undoManager];
-    /* save the method on the undo stack */
-    [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
-    [uMgr setActionName:@"Create Circle"];
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Create Circle"];
     
-    [self saveCurrentObjects];
+  [self saveCurrentObjects];
 
-    for(i = 0; i < [objects count]; i++)
+
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if([obj isKindOfClass: [GRCircle class]])
-            if(![[obj editor] isdone])
-                isneweditor = NO;
+      GRObjectEditor *objEdi;
+
+      objEdi = [[objects objectAtIndex: i] editor];
+      if (![objEdi isSelect])
+        [objEdi unselect];
     }
 
-    if(isneweditor)
-        for(i = 0; i < [objects count]; i++)
-        {
-            GRObjectEditor *objEdi;
+  nextEvent = [[self window] nextEventMatchingMask:
+                               NSLeftMouseUpMask | NSLeftMouseDraggedMask];
+  [self verifyModifiersOfEvent: nextEvent];
 
-            objEdi = [[objects objectAtIndex: i] editor];
-            if (![objEdi isSelect])
-                [objEdi unselect];
-        }
-
-            nextEvent = [[self window] nextEventMatchingMask:
-                NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-    [self verifyModifiersOfEvent: nextEvent];
-
-    if([nextEvent type] != NSLeftMouseDragged)
+  if([nextEvent type] != NSLeftMouseDragged)
     {
-        NSLog(@"is not left mouse dragged");
-        if(isneweditor)
-        {
-            [self addBox];
-            circle = [objects objectAtIndex: edind];
-            [[circle editor] selectForEditing];
-            [circle setStartAtPoint: p];
-            [self setNeedsDisplay: YES];
-            return;
-        } else
-        {
-            circle = [objects objectAtIndex: edind];
-            [[circle editor] selectForEditing];
-            //            if(shiftclick)
-            //                p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
-            [circle setEndAtPoint: p];
-            [self setNeedsDisplay: YES];
-            return;
-        }
-    } else
+      NSLog(@"is not left mouse dragged");
+
+      [self addBox];
+      circle = [objects objectAtIndex: edind];
+      [[circle editor] selectForEditing];
+      [circle setStartAtPoint: p];
+      [self setNeedsDisplay: YES];
+      return;
+    }
+  else
     {
-        NSLog(@"is left mouse dragged");
-        if(isneweditor)
-        {
-            NSLog(@"is new editor");
-            [self addCircle];
-            circle = [objects objectAtIndex: edind];
-            [[circle editor] selectForEditing];
-            [circle setStartAtPoint: p];
-            [self setNeedsDisplay: YES];
-        } else
-        {
-            NSLog(@"is old editor");
-            circle = [objects objectAtIndex: edind];
-            [[circle editor] selectForEditing];
-            //            if(shiftclick)
-            //                p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
-            [circle setEndAtPoint: p];
-            [self setNeedsDisplay: YES];
-        }
+      NSLog(@"is left mouse dragged");
 
-        do
-        {
-            p = [nextEvent locationInWindow];
-            p = [self convertPoint: p fromView: nil];
-            //            if(shiftclick)
-            //                p = pointApplyingCostrainerToPoint(p, [[bzpath lastPoint] center]);
+      [self addCircle];
+      circle = [objects objectAtIndex: edind];
+      [[circle editor] selectForEditing];
+      [circle setStartAtPoint: p];
+      [circle setEndAtPoint: p];
+      [self setNeedsDisplay: YES];
 
-            [circle setEndAtPoint: p];
 
-            [self setNeedsDisplay: YES];
+      [self moveControlPointOfEditor: (GRBezierPathEditor *)[circle editor] toPoint: p];
+
+      [[circle editor] unselect];
+      [[circle editor] selectAsGroup];
 
-            nextEvent = [[self window] nextEventMatchingMask:
-                NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-            [self verifyModifiersOfEvent: nextEvent];
-        } while([nextEvent type] != NSLeftMouseUp);
-        if (isneweditor)
-        {
-            [[circle editor] unselect];
-            [[circle editor] selectAsGroup];
-        }
     }
 }
 
 
 - (void)selectObjectAtPoint:(NSPoint)p
 {
-    id obj;
-    NSMutableArray *objs;
-    int i;
+  id obj;
+  NSMutableArray *objs;
+  NSUInteger i;
 
-    objs = [NSMutableArray arrayWithCapacity: 1];
+  objs = [NSMutableArray arrayWithCapacity: 1];
 
-    for(i = 0; i < [objects count]; i++)
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if([obj isKindOfClass: [GRBezierPath class]])
+      obj = [objects objectAtIndex: i];
+
+      if([obj objectHitForSelection: p])
         {
-            if([obj onPathBorder: p])
-                [[obj editor] selectAsGroup];
-            else
-                if(!shiftclick)
-                    [[obj editor] unselect];
-        } else
-        {
-            if([obj pointInBounds: p])
-                [[obj editor] select];
-            else if(!shiftclick)
-                [[obj editor] unselect];
+          if (shiftclick && [[obj editor] isSelect])
+            [[obj editor] unselect];
+          else
+            [[obj editor] select];
         }
+      else if(!shiftclick)
+	[[obj editor] unselect];
     }
 
-    for(i = 0; i < [objects count]; i++)
+  for(i = 0; i < [objects count]; i++)
     {
         obj = [objects objectAtIndex: i];
         if([[obj editor] isGroupSelected])
@@ -812,7 +715,7 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 - (void)editPathAtPoint:(NSPoint)p
 {
     id obj;
-    int i;
+    NSUInteger i;
     NSUndoManager *uMgr;
     
     uMgr = [self undoManager];
@@ -860,9 +763,9 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)editTextAtPoint:(NSPoint)p
 {
-    id obj;
-    int i;
-    NSUndoManager *uMgr;
+  id obj;
+  NSUInteger i;
+  NSUndoManager *uMgr;
     
     uMgr = [self undoManager];
     /* save the method on the undo stack */
@@ -879,7 +782,7 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
         obj = [objects objectAtIndex: i];
         if([obj isKindOfClass: [GRText class]])
         {
-            if([obj pointInBounds: p])
+            if([obj objectHitForSelection: p])
             {
                 [[obj editor] select];
                 [self setNeedsDisplay: YES];
@@ -894,7 +797,7 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 - (void)editSelectedText
 {
     id obj;
-    int i;
+    NSUInteger i;
 
     for(i = 0; i < [objects count]; i++)
     {
@@ -909,13 +812,13 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)moveSelectedObjects:(NSArray *)objs startingPoint:(NSPoint)startp
 {
-    NSEvent *nextEvent;
-    NSArray *moveobjs = nil;
-    id obj;
-    NSPoint p, op, diffp;
-    BOOL dupl = NO;
-    int i;
-    NSUndoManager *uMgr;
+  NSEvent *nextEvent;
+  NSArray *moveobjs = nil;
+  id obj;
+  NSPoint p, op, diffp;
+  BOOL dupl = NO;
+  NSUInteger i;
+  NSUndoManager *uMgr;
     
     uMgr = [self undoManager];
     /* save the method on the undo stack */
@@ -976,6 +879,7 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 }
 
 
+/* propagates control point editing down to each editor */
 - (BOOL)moveControlPointOfEditor:(GRPathEditor *)editor toPoint:(NSPoint)pos
 {
     NSPoint p;
@@ -1002,8 +906,8 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)subdividePathAtPoint:(NSPoint)p splitIt:(BOOL)split
 {
-    id obj;
-    int i;
+  id obj;
+  NSUInteger i;
 
     for(i = 0; i < [objects count]; i++)
     {
@@ -1023,10 +927,10 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 - (NSDictionary *)selectionProperties
 {
   NSMutableDictionary *propDict;
-  int i;
-  int selectedObjects;
-  int pathObjNum;
-  int textObjNum;
+  NSUInteger i;
+  NSUInteger selectedObjects;
+  NSUInteger pathObjNum;
+  NSUInteger textObjNum;
   NSNumber *num;
   
   if(![objects count])
@@ -1045,10 +949,10 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
         {
           selectedObjects++;
         
-          num = [NSNumber numberWithInt: [obj isStroked]];
+          num = [NSNumber numberWithBool: [obj isStroked]];
           [propDict setObject: num forKey: @"stroked"];
           [propDict setObject: [obj strokeColor] forKey: @"strokecolor"];
-          num = [NSNumber numberWithInt: [obj isFilled]];
+          num = [NSNumber numberWithBool: [obj isFilled]];
           [propDict setObject: num forKey: @"filled"];
           [propDict setObject: [obj fillColor] forKey: @"fillcolor"];
 
@@ -1065,7 +969,6 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
               [propDict setObject: num forKey: @"miterlimit"];
               num = [NSNumber numberWithFloat: [obj lineWidth]];
               [propDict setObject: num forKey: @"linewidth"];
-              num = [NSNumber numberWithInt: [obj isStroked]];
             }
           else if([obj isKindOfClass: [GRText class]])
             {
@@ -1101,7 +1004,7 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 {
   NSUndoManager *uMgr;
   id obj;
-  int i;
+  NSUInteger i;
   
   uMgr = [self undoManager];
   /* save the method on the undo stack, but stack actions */
@@ -1127,10 +1030,10 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
               [obj setMiterLimit: [[properties objectForKey: @"miterlimit"] floatValue]];
               [obj setLineWidth: [[properties objectForKey: @"linewidth"] floatValue]];
             }
-          [obj setStroked: (BOOL)[[properties objectForKey: @"stroked"] intValue]];
+          [obj setStroked: [[properties objectForKey: @"stroked"] boolValue]];
           newColor = (NSColor *)[properties objectForKey: @"strokecolor"];
           [obj setStrokeColor: newColor];
-          [obj setFilled: (BOOL)[[properties objectForKey: @"filled"] intValue]];
+          [obj setFilled: [[properties objectForKey: @"filled"] boolValue]];
           newColor = (NSColor *)[properties objectForKey: @"fillcolor"];
           [obj setFillColor: newColor];
         }
@@ -1140,9 +1043,9 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)moveSelectedObjectsToFront:(id)sender
 {
-    id obj = nil;
-    int i;
-    NSUndoManager *uMgr;
+  id obj = nil;
+  NSUInteger i;
+  NSUndoManager *uMgr;
 
     uMgr = [self undoManager];
     /* save the method on the undo stack */
@@ -1182,9 +1085,9 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)moveSelectedObjectsToBack:(id)sender
 {
-    id obj = nil;
-    int i;
-    NSUndoManager *uMgr;
+  id obj = nil;
+  NSUInteger i;
+  NSUndoManager *uMgr;
 
     uMgr = [self undoManager];
     /* save the method on the undo stack */
@@ -1210,7 +1113,7 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
     for(i = 0; i < [objects count]; i++)
     {
-        if((obj == [objects objectAtIndex: i]) && ((i - 1) >= 0)) 
+        if((obj == [objects objectAtIndex: i]) && (i > 0)) 
         {
             [objects removeObjectAtIndex: i];
             [objects insertObject: obj atIndex: i - 1];
@@ -1241,50 +1144,153 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)zoomOnPoint:(NSPoint)p zoomOut:(BOOL)isout
 {
-    float orx, ory, szx, szy;
-    NSRect vr;
-    NSPoint pp;
-    int i;
+  NSUInteger i;
 
-    if(isout)
+  i = zIndex;
+  if(isout)
     {
-        zIndex--;
-        if(zIndex < 0)
-        {
-            zIndex = 0;
-            return;
-        }
-    } else
+      if (i == 0)
+	return;
+      i--; 
+    }
+  else
     {
-        zIndex++;
-        if(zIndex > 8)
-        {
-            zIndex = 8;
-            return;
-        }
+      if (i == ZOOM_FACTORS-1)
+	return;
+      i++;
     }
 
-    zFactor = zFactors[zIndex];
+  [self zoomOnPoint:p withFactor:i];
+}
 
-    orx = a4Rect.origin.x * zFactor;
-    ory = a4Rect.origin.y * zFactor;
-    szx = a4Rect.size.width * zFactor;
-    szy = a4Rect.size.height * zFactor;
-    zmdRect = NSMakeRect(orx, ory, szx, szy);
-    szx = pageRect.size.width * zFactor;
-    szy = pageRect.size.height * zFactor;
 
-    pp.x = p.x * ([self frame].origin.x + pageRect.size.width) / [self frame].size.width;
-    pp.y = p.y * ([self frame].origin.y + pageRect.size.height) / [self frame].size.height;
-    vr = NSMakeRect(pp.x * zFactor - 200, pp.y * zFactor - 200, 400, 400);
+- (void)zoomOnPoint:(NSPoint)p withFactor:(int)index
+{
+  CGFloat orx, ory, szx, szy;
+  NSRect vr;
+  NSPoint pp;
+  NSUInteger i;
 
-    [self setFrame: NSMakeRect(0, 0, szx, szy)];
-    [self scrollRectToVisible: vr];
+  zIndex = index;
+  zFactor = zFactors[zIndex];
 
-    for(i = 0; i < [objects count]; i++)
-        [[objects objectAtIndex: i] setZoomFactor: zFactor];
+  orx = a4Rect.origin.x * zFactor;
+  ory = a4Rect.origin.y * zFactor;
+  szx = a4Rect.size.width * zFactor;
+  szy = a4Rect.size.height * zFactor;
+  zmdRect = NSMakeRect(orx, ory, szx, szy);
+  szx = pageRect.size.width * zFactor;
+  szy = pageRect.size.height * zFactor;
 
-    [[self window] display];
+  pp.x = p.x * ([self frame].origin.x + pageRect.size.width) / [self frame].size.width;
+  pp.y = p.y * ([self frame].origin.y + pageRect.size.height) / [self frame].size.height;
+  vr = NSMakeRect(pp.x * zFactor - 200, pp.y * zFactor - 200, 400, 400);
+  [self setFrame: NSMakeRect(0, 0, szx, szy)];
+  [self scrollRectToVisible: vr];
+
+  for(i = 0; i < [objects count]; i++)
+    [[objects objectAtIndex: i] setZoomFactor: zFactor];
+
+  [[self window] display];
+}
+
+- (IBAction)zoom50:(id)sender
+{
+  NSUInteger i;
+  NSPoint p;
+  NSRect visibleRect;
+
+  zFactor = 0.5;
+
+  i = ZOOM_FACTORS - 1;
+  while (i > 0 && zFactor < zFactors[i])
+    i--;
+
+  visibleRect = [self visibleRect];
+  p.x = NSMinX(visibleRect) + NSWidth(visibleRect) / 2;
+  p.y = NSMinY(visibleRect) + NSHeight(visibleRect) / 2;
+
+  [self zoomOnPoint:p withFactor:i];
+}
+
+- (IBAction)zoom100:(id)sender
+{
+  NSUInteger  i;
+  NSPoint p;
+  NSRect visibleRect;
+
+  zFactor = 1;
+
+  i = ZOOM_FACTORS - 1;
+  while (i > 0 && zFactor < zFactors[i])
+    i--;
+
+  visibleRect = [self visibleRect];
+  p.x = NSMinX(visibleRect) + NSWidth(visibleRect) / 2;
+  p.y = NSMinY(visibleRect) + NSHeight(visibleRect) / 2;
+
+  [self zoomOnPoint:p withFactor:i];
+}
+
+- (IBAction)zoom200:(id)sender
+{
+  NSUInteger i;
+  NSPoint p;
+  NSRect visibleRect;
+
+  zFactor = 2;
+
+  i = ZOOM_FACTORS - 1;
+  while (i > 0 && zFactor < zFactors[i])
+    i--;
+
+  visibleRect = [self visibleRect];
+  p.x = NSMinX(visibleRect) + NSWidth(visibleRect) / 2;
+  p.y = NSMinY(visibleRect) + NSHeight(visibleRect) / 2;
+
+  [self zoomOnPoint:p withFactor:i];
+}
+
+- (IBAction)zoomFitPage:(id)sender
+{
+  NSUInteger i;
+  NSPoint p;
+  NSRect visibleRect;
+  NSRect f;
+
+  f = [self frame];
+  visibleRect = [self visibleRect];
+  i = ZOOM_FACTORS - 1;
+  NSLog(@"visible %f, full %f", visibleRect.size.width, f.size.width);
+  while (i > 0 && ((visibleRect.size.width < f.size.width * zFactors[i]) || (visibleRect.size.height < f.size.height * zFactors[i])))
+    i--;
+
+  visibleRect = [self visibleRect];
+  p.x = NSMinX(visibleRect) + NSWidth(visibleRect) / 2;
+  p.y = NSMinY(visibleRect) + NSHeight(visibleRect) / 2;
+
+  [self zoomOnPoint:p withFactor:i];
+}
+
+- (IBAction)zoomFitWidth:(id)sender
+{
+  NSUInteger i;
+  NSPoint p;
+  NSRect visibleRect;
+  NSRect f;
+
+  f = [self frame];
+  visibleRect = [self visibleRect];
+  i = ZOOM_FACTORS - 1;
+  NSLog(@"visible %f, full %f", visibleRect.size.width, f.size.width);
+  while (i > 0 && visibleRect.size.width < f.size.width * zFactors[i])
+    i--;
+ 
+  visibleRect = [self visibleRect];
+  p.x = NSMinX(visibleRect) + NSWidth(visibleRect) / 2;
+  p.y = NSMinY(visibleRect) + NSHeight(visibleRect) / 2;
+
+  [self zoomOnPoint:p withFactor:i];
 }
 
 - (void)movePageFromHandPoint:(NSPoint)handpos
@@ -1313,52 +1319,82 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
     }
 }
 
+- (void)delete:(id)sender
+{
+  NSUndoManager *uMgr;
+
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Delete"];
+  
+  [self saveCurrentObjects];
+  
+  [self deleteSelectedObjects];
+}
+
 - (void)cut:(id)sender
 {
-    [self copy: sender];
-    [self deleteSelectedObjects];
+  NSUndoManager *uMgr;
+
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Cut"];
+  
+  [self saveCurrentObjects];
+  [self copy: sender];
+  [self deleteSelectedObjects];
 }
 
 - (void)copy:(id)sender
 {
-    NSMutableArray *types;
-    NSPasteboard *pboard;
-    id obj;
-    NSMutableArray *objsdesc;
-    int i;
+  NSMutableArray *types;
+  NSPasteboard *pboard;
+  id obj;
+  NSMutableArray *objsdesc;
+  NSUInteger i;
 
-    objsdesc = [NSMutableArray arrayWithCapacity: 1];
-    for(i = 0; i < [objects count]; i++)
+  objsdesc = [NSMutableArray arrayWithCapacity: 1];
+  for(i = 0; i < [objects count]; i++)
     {
-        obj = [objects objectAtIndex: i];
-        if([[obj editor] isGroupSelected])
-            [objsdesc addObject: [obj objectDescription]];
+      obj = [objects objectAtIndex: i];
+      if([[obj editor] isGroupSelected])
+	[objsdesc addObject: [obj objectDescription]];
     }
-
-    if([objsdesc count])
+  
+  if([objsdesc count])
     {
-        types = [NSMutableArray arrayWithObjects: @"GRObjectPboardType", nil];
-        pboard = [NSPasteboard generalPasteboard];
-        [pboard declareTypes: types owner: self];
-        [pboard setString: [objsdesc description] forType: @"GRObjectPboardType"];
+      types = [NSMutableArray arrayWithObjects: @"GRObjectPboardType", nil];
+      pboard = [NSPasteboard generalPasteboard];
+      [pboard declareTypes: types owner: self];
+      [pboard setString:[objsdesc description] forType: @"GRObjectPboardType"];
     }
 }
 
 - (void)paste:(id)sender
 {
-    NSPasteboard *pboard;
-    NSArray *types;
-    NSArray *descriptions;
-    NSDictionary *objdesc;
-    id obj;
-    NSString *str;
-    int i;
+  NSPasteboard *pboard;
+  NSArray *types;
+  NSArray *descriptions;
+  NSDictionary *objdesc;
+  id obj;
+  NSString *str;
+  NSUInteger i;
+  NSUndoManager *uMgr;
+  
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Paste"];
+  
+  [self saveCurrentObjects];
 
-    pboard = [NSPasteboard generalPasteboard];
-    types = [NSArray arrayWithObject: @"GRObjectPboardType"];
-    if([[pboard availableTypeFromArray: types] isEqualToString: @"GRObjectPboardType"])
+  pboard = [NSPasteboard generalPasteboard];
+  types = [NSArray arrayWithObject: @"GRObjectPboardType"];
+  if([[pboard availableTypeFromArray: types] isEqualToString: @"GRObjectPboardType"])
     {
-        descriptions = (NSArray *)[[pboard stringForType: @"GRObjectPboardType"] propertyList];
+        descriptions = [[pboard stringForType: @"GRObjectPboardType"] propertyList];
 
         for(i = 0; i < [descriptions count]; i++)
         {
@@ -1393,39 +1429,39 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (NSMutableArray *)deepCopyObjects: (NSMutableArray *)objArray
 {
-    NSMutableArray *copyArray;
-    NSEnumerator *e;
-    NSObject *o;
+  NSMutableArray *copyArray;
+  NSEnumerator *e;
+  NSObject *o;
 
     
-    copyArray = [[NSMutableArray arrayWithCapacity:[objArray count]] retain];
+  copyArray = [NSMutableArray arrayWithCapacity:[objArray count]];
 
-    e = [objArray objectEnumerator];
-    while ((o = [e nextObject]))
+  e = [objArray objectEnumerator];
+  while ((o = [e nextObject]))
     {
-        [copyArray addObject:[o copy]];
+      [copyArray addObject:[[o copy] autorelease]];
     }
 
-    return copyArray;
+  return copyArray;
 }
 
 - (void)saveCurrentObjects
 {
-    if (objects != nil)
+  if (objects != nil)
     {
-        if (lastObjects != nil)
-            [lastObjects release];
-        lastObjects = [[NSMutableArray arrayWithArray:objects] retain];
+      if (lastObjects != nil)
+        [lastObjects release];
+      lastObjects = [[NSMutableArray arrayWithArray:objects] retain];
     }
 }
 
 - (void)saveCurrentObjectsDeep
 {
-    if (objects != nil)
+  if (objects != nil)
     {
-        if (lastObjects != nil)
-            [lastObjects release];
-        lastObjects = [self deepCopyObjects: objects];
+      if (lastObjects != nil)
+        [lastObjects release];
+      lastObjects = [[self deepCopyObjects: objects] retain];
     }
 }
 
@@ -1499,17 +1535,17 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)mouseDown:(NSEvent *)theEvent
 {
-    NSPoint p;
-    int count = [theEvent clickCount];
+  NSPoint p;
+  NSUInteger count = [theEvent clickCount];
 
-    [self verifyModifiersOfEvent: theEvent];
+  [self verifyModifiersOfEvent: theEvent];
 
-    p = [theEvent locationInWindow];
-    p = [self convertPoint: p fromView: nil];
+  p = [theEvent locationInWindow];
+  p = [self convertPoint: p fromView: nil];
 
-    if(count == 1)
+  if(count == 1)
     {
-        switch([[NSApp delegate] currentToolType])
+      switch([[NSApp delegate] currentToolType])
         {
             case blackarrowtool:
                 [self selectObjectAtPoint: p];
@@ -1591,10 +1627,10 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)keyDown:(NSEvent *)theEvent
 {
-    unsigned short keyCode;
-    NSRect vRect, hiddRect;
-    NSPoint vPoint;
-    float hiddRx, hiddRy, hiddRw, hiddRh;
+  unsigned short keyCode;
+  NSRect vRect, hiddRect;
+  NSPoint vPoint;
+  CGFloat hiddRx, hiddRy, hiddRw, hiddRh;
 
     keyCode = [theEvent keyCode];
 
@@ -1636,9 +1672,9 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
 
 - (void)drawRect:(NSRect)rect
 {
-    int          i;
-    NSRect       frameRect;
-    NSBezierPath *bzp;
+  NSUInteger   i;
+  NSRect       frameRect;
+  NSBezierPath *bzp;
 
     frameRect = [self frame];
     
@@ -1671,17 +1707,17 @@ float zFactors[9] = {0.25, 0.5, 1, 1.5, 2, 3, 4, 6, 8};
  */
 - (BOOL) knowsPageRange: (NSRangePointer) range
 {
-    /* we simply set one page */
-    range->location = 1;
-    range->length = 1;
+  /* we simply set one page */
+  range->location = 1;
+  range->length = 1;
     
-    return YES;
+  return YES;
 }
 
 /**
  * override for a custom pagination scheme
  */
-- (NSRect ) rectForPage: (int) pageNumber
+- (NSRect ) rectForPage: (NSInteger) pageNumber
 {
     NSRect pageRec;
     NSSize pageSize;
