@@ -1,8 +1,8 @@
 /*
  Project: Graphos
- GRDrawableObject.h
+ GRDrawableObject.m
 
- Copyright (C) 2008-2010 GNUstep Application Project
+ Copyright (C) 2008-2013 GNUstep Application Project
 
  Author: Ing. Riccardo Mottola
 
@@ -38,6 +38,65 @@
     [super dealloc];
 }
 
+/** initializes by using the properties array as defaults */
+- (id)initInView:(GRDocView *)aView
+      zoomFactor:(CGFloat)zf
+      withProperties:(NSDictionary *)properties
+{
+  self = [super init];
+  if(self)
+    {
+      NSColor *newColor;
+      id val;
+
+      NSLog(@"initInView properties of GRDrawableObject");
+
+      docView = aView;
+      zmFactor = zf;
+      stroked = YES;
+      filled = NO;
+      visible = YES;
+      locked = NO;
+      strokeColor = [[[NSColor blackColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+      fillColor = [[[NSColor whiteColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+      
+      val = [properties objectForKey: @"stroked"];
+      if (val != nil)
+	[self setStroked: [val boolValue]];
+      newColor = (NSColor *)[properties objectForKey: @"strokecolor"];
+      if (newColor != nil)
+	[self setStrokeColor: newColor];
+
+      val = [properties objectForKey: @"filled"];
+      if (val != nil)
+	[self setFilled: [val boolValue]];
+      newColor = (NSColor *)[properties objectForKey: @"fillcolor"];
+      if (newColor != nil)
+	[self setFillColor: newColor];
+
+      val = [properties objectForKey: @"visible"];
+      if (val)	
+	visible = [val boolValue];
+      
+      val = [properties objectForKey: @"locked"];
+      if (val)
+	locked = [val boolValue];
+    }
+  return self;
+}
+
+/** initializes all parameters from a description dictionary */
+- (id)initFromData:(NSDictionary *)description
+            inView:(GRDocView *)aView
+        zoomFactor:(CGFloat)zf
+{
+#ifdef GNUSTEP
+  [self subclassResponsibility: _cmd];
+#endif
+  return nil;
+}
+
+
 - (NSDictionary *)objectDescription
 {
 #ifdef GNUSTEP
@@ -65,6 +124,13 @@
     return objCopy;
 }
 
+- (BOOL)objectHitForSelection:(NSPoint)p
+{
+#ifdef GNUSTEP
+    [self subclassResponsibility: _cmd];
+#endif
+    return NO;
+}
 
 - (GRDocView *)view
 {
@@ -98,7 +164,7 @@
     locked = value;
 }
 
-- (void)setZoomFactor:(float)f
+- (void)setZoomFactor:(CGFloat)f
 {
     zmFactor = f;
 }
