@@ -2,7 +2,7 @@
  Project: Graphos
  GRObjectControlPoint.h
 
- Copyright (C) 2007-2008 GNUstep Application Project
+ Copyright (C) 2007-2013 GNUstep Application Project
 
  Author: Ing. Riccardo Mottola
 
@@ -25,6 +25,12 @@
 
 #import <Foundation/Foundation.h>
 
+#if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4) && !defined(CGFloat)
+#define NSUInteger unsigned
+#define NSInteger int
+#define CGFloat float
+#endif
+
 @class GRBoxEditor;
 
 
@@ -35,14 +41,14 @@
     NSPoint center;
     NSRect centerRect;
     NSRect innerRect;
-    float zmFactor;
+    CGFloat zmFactor;
 }
 
-- (id)initAtPoint:(NSPoint)aPoint zoomFactor:(float)zf;
+- (id)initAtPoint:(NSPoint)aPoint zoomFactor:(CGFloat)zf;
 
 - (void)moveToPoint:(NSPoint)p;
 
-- (void)setZoomFactor:(float)f;
+- (void)setZoomFactor:(CGFloat)f;
 
 - (NSPoint)center;
 - (NSRect)centerRect;
