@@ -2,7 +2,7 @@
  Project: Graphos
  GRText.m
 
- Copyright (C) 2000-2011 GNUstep Application Project
+ Copyright (C) 2000-2013 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -35,7 +35,7 @@
 
 - (id)initInView:(GRDocView *)aView
          atPoint:(NSPoint)p
-      zoomFactor:(float)zf
+      zoomFactor:(CGFloat)zf
   withProperties:(NSDictionary *) properties
       openEditor:(BOOL)openedit
 {
@@ -54,18 +54,18 @@
       filled = NO;
       visible = YES;
       locked = NO;
-      strokeColor = [[NSColor blackColor] retain];
-      fillColor = [[NSColor whiteColor] retain];
+      strokeColor = [[[NSColor blackColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+      fillColor = [[[NSColor whiteColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
       ASSIGN(str, @"");
 
       if (properties != nil)
 	{
 	  NSColor *newColor;
-	  NSString *val;
+	  id val;
 	  
 	  val = [properties objectForKey: @"stroked"];
 	  if (val != nil)
-	    [self setStroked: (BOOL)[val intValue]];
+	    [self setStroked:[val boolValue]];
 	  newColor = (NSColor *)[properties objectForKey: @"strokecolor"];
 	  if (newColor != nil)
 	    [self setStrokeColor: newColor];
@@ -85,8 +85,15 @@
 				attributes: nil];
 	  result = [(GRTextEditor *)editor runModal];
 	  if(result == NSAlertDefaultReturn)
-	    [self setString: [[(GRTextEditor *)editor editorView] textString]
-		 attributes: [[(GRTextEditor *)editor editorView] textAttributes]];
+	    {
+	      [self setString: [[(GRTextEditor *)editor editorView] textString]
+		   attributes: [[(GRTextEditor *)editor editorView] textAttributes]];
+	    }
+	  else
+	    {
+	      [self release];
+	      return nil;
+	    }
         }
     }
   return self;
@@ -94,7 +101,7 @@
 
 - (id)initFromData:(NSDictionary *)description
             inView:(GRDocView *)aView
-        zoomFactor:(float)zf
+        zoomFactor:(CGFloat)zf
 {
     NSMutableParagraphStyle *style;
     NSDictionary *attrs;
@@ -108,7 +115,8 @@
         float fillCol[4];
 	float strokeAlpha;
 	float fillAlpha;
-
+	id obj;
+	
         docView = aView;
         zmFactor = zf;
         editor = [[GRTextEditor alloc] initEditor:self];
@@ -128,44 +136,82 @@
         attrs = [NSDictionary dictionaryWithObjectsAndKeys:
             font, NSFontAttributeName,
             style, NSParagraphStyleAttributeName, nil];
+	[style release];
         size = [str sizeWithAttributes: attrs];
         scalex = [[description objectForKey: @"scalex"] floatValue];
         scaley = [[description objectForKey: @"scaley"] floatValue];
         rotation = [[description objectForKey: @"rotation"] floatValue];
 
-        stroked = (BOOL)[[description objectForKey: @"stroked"] intValue];
+	obj = [description objectForKey: @"stroked"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        stroked = [obj boolValue];
+        strokeAlpha = [[description objectForKey: @"strokealpha"] floatValue];
         s = [description objectForKey: @"strokecolor"];
         linearr = [s componentsSeparatedByString: @" "];
-        strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
-        strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
-        strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
-        strokeCol[3] = [[linearr objectAtIndex: 3] floatValue];
-        strokeAlpha = [[description objectForKey: @"strokealpha"] floatValue];
-	strokeColor = [NSColor colorWithDeviceCyan: strokeCol[0]
-					   magenta: strokeCol[1]
-					    yellow: strokeCol[2]
-					     black: strokeCol[3]
-					     alpha: strokeAlpha];
-	strokeColor = [strokeColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
-
-        filled = (BOOL)[[description objectForKey: @"filled"] intValue];
+	if ([linearr count] == 3)
+	  {
+	    strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    strokeColor = [NSColor colorWithCalibratedRed: strokeCol[0]
+				   green: strokeCol[1]
+				   blue: strokeCol[2]
+				   alpha: strokeAlpha];
+	    [strokeColor retain];
+	  }
+	else
+	  {
+	    strokeCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    strokeCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    strokeCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    strokeCol[3] = [[linearr objectAtIndex: 3] floatValue];
+	    strokeColor = [NSColor colorWithDeviceCyan: strokeCol[0]
+				   magenta: strokeCol[1]
+				   yellow: strokeCol[2]
+				   black: strokeCol[3]
+				   alpha: strokeAlpha];
+	    strokeColor = [[strokeColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+	  }
+	obj = [description objectForKey: @"filled"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        filled = [obj boolValue];
+	fillAlpha = [[description objectForKey: @"fillalpha"] floatValue];
         s = [description objectForKey: @"fillcolor"];
         linearr = [s componentsSeparatedByString: @" "];
-        fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
-        fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
-        fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
-        fillCol[3] = [[linearr objectAtIndex: 3] floatValue];
-        fillAlpha = [[description objectForKey: @"fillalpha"] floatValue];
-        fillAlpha = [[description objectForKey: @"fillalpha"] floatValue];
-	fillColor = [NSColor colorWithDeviceCyan: fillCol[0]
-					 magenta: fillCol[1]
-					  yellow: fillCol[2]
-					   black: fillCol[3]
-					   alpha: fillAlpha];
-	fillColor = [fillColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
-
-        visible = (BOOL)[[description objectForKey: @"visible"] intValue];
-        locked = (BOOL)[[description objectForKey: @"locked"] intValue];
+	if ([linearr count] == 3)
+	  {
+	    fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    fillColor = [NSColor colorWithCalibratedRed: fillCol[0]
+				 green: fillCol[1]
+				 blue: fillCol[2]
+				 alpha: fillAlpha];
+	    [fillColor retain];
+	  }
+	else
+	  {
+	    fillCol[0] = [[linearr objectAtIndex: 0] floatValue];
+	    fillCol[1] = [[linearr objectAtIndex: 1] floatValue];
+	    fillCol[2] = [[linearr objectAtIndex: 2] floatValue];
+	    fillCol[3] = [[linearr objectAtIndex: 3] floatValue];
+	    fillColor = [NSColor colorWithDeviceCyan: fillCol[0]
+				 magenta: fillCol[1]
+				 yellow: fillCol[2]
+				 black: fillCol[3]
+				 alpha: fillAlpha];
+	    fillColor = [[fillColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+	  }
+	obj = [description objectForKey: @"visible"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        visible = [obj boolValue];
+	obj = [description objectForKey: @"locked"];
+	if ([obj isKindOfClass:[NSString class]])
+	  obj = [NSNumber numberWithInt:[obj intValue]];
+        locked = [obj boolValue];
 
         zmFactor = [[description objectForKey: @"zmfactor"] floatValue];
         [self setZoomFactor: zf];
@@ -191,26 +237,20 @@
 {
     NSMutableDictionary *dict;
     NSString *s;
-    NSColor *strokeColorCMYK;
-    NSColor *fillColorCMYK;
-    float strokeCol[4];
-    float fillCol[4];
+    float strokeCol[3];
+    float fillCol[3];
     float strokeAlpha;
     float fillAlpha;
 
-    strokeColorCMYK = [strokeColor colorUsingColorSpaceName: NSDeviceCMYKColorSpace]; 
-    strokeCol[0] = [strokeColorCMYK cyanComponent];
-    strokeCol[1] = [strokeColorCMYK magentaComponent];
-    strokeCol[2] = [strokeColorCMYK yellowComponent];
-    strokeCol[3] = [strokeColorCMYK blackComponent];
-    strokeAlpha = [strokeColorCMYK alphaComponent];
+    strokeCol[0] = [strokeColor redComponent];
+    strokeCol[1] = [strokeColor greenComponent];
+    strokeCol[2] = [strokeColor blueComponent];
+    strokeAlpha = [strokeColor alphaComponent];
 
-    fillColorCMYK = [fillColor colorUsingColorSpaceName: NSDeviceCMYKColorSpace]; 
-    fillCol[0] = [fillColorCMYK cyanComponent];
-    fillCol[1] = [fillColorCMYK magentaComponent];
-    fillCol[2] = [fillColorCMYK yellowComponent];
-    fillCol[3] = [fillColorCMYK blackComponent];
-    fillAlpha = [fillColorCMYK alphaComponent];
+    fillCol[0] = [fillColor redComponent];
+    fillCol[1] = [fillColor greenComponent];
+    fillCol[2] = [fillColor blueComponent];
+    fillAlpha = [fillColor alphaComponent];
 
     dict = [NSMutableDictionary dictionaryWithCapacity: 1];
     [dict setObject: @"text" forKey: @"type"];
@@ -234,24 +274,20 @@
     [dict setObject: s forKey: @"scaley"];
     s = [NSString stringWithFormat: @"%.3f", rotation];
     [dict setObject: s forKey: @"rotation"];
-    s = [NSString stringWithFormat: @"%i", stroked];
-    [dict setObject: s forKey: @"stroked"];
-    s = [NSString stringWithFormat: @"%.3f %.3f %.3f %.3f",
-        strokeCol[0], strokeCol[1], strokeCol[2], strokeCol[3]];
+    [dict setObject: [NSNumber numberWithBool:stroked] forKey: @"stroked"];
+    s = [NSString stringWithFormat: @"%.3f %.3f %.3f",
+        strokeCol[0], strokeCol[1], strokeCol[2]];
     [dict setObject: s forKey: @"strokecolor"];
     s = [NSString stringWithFormat: @"%.3f", strokeAlpha];
     [dict setObject: s forKey: @"strokealpha"];
-    s = [NSString stringWithFormat: @"%i", filled];
-    [dict setObject: s forKey: @"filled"];
-    s = [NSString stringWithFormat: @"%.3f %.3f %.3f %.3f",
-        fillCol[0], fillCol[1], fillCol[2], fillCol[3]];
+    [dict setObject: [NSNumber numberWithBool: filled] forKey: @"filled"];
+    s = [NSString stringWithFormat: @"%.3f %.3f %.3f",
+        fillCol[0], fillCol[1], fillCol[2]];
     [dict setObject: s forKey: @"fillcolor"];
     s = [NSString stringWithFormat: @"%.3f", fillAlpha];
     [dict setObject: s forKey: @"fillalpha"];
-    s = [NSString stringWithFormat: @"%i", visible];
-    [dict setObject: s forKey: @"visible"];
-    s = [NSString stringWithFormat: @"%i", locked];
-    [dict setObject: s forKey: @"locked"];
+    [dict setObject: [NSNumber numberWithBool: visible] forKey: @"visible"];
+    [dict setObject: [NSNumber numberWithBool: locked] forKey: @"locked"];
     s = [NSString stringWithFormat: @"%.3f", zmFactor];
     [dict setObject: s forKey: @"zmfactor"];
 
@@ -299,6 +335,7 @@
     attrs = [NSDictionary dictionaryWithObjectsAndKeys:
         font, NSFontAttributeName,
         style, NSParagraphStyleAttributeName, nil];
+    [style release];
 
     [(GRTextEditor *)editor setPoint: pos
           withString: str
@@ -309,7 +346,7 @@
              attributes: [[(GRTextEditor *)editor editorView] textAttributes]];
 }
 
-- (BOOL)pointInBounds:(NSPoint)p
+- (BOOL)objectHitForSelection:(NSPoint)p
 {
     if(pointInRect(bounds, p) || pointInRect(selRect, p))
         return YES;
@@ -324,7 +361,7 @@
     selRect = NSMakeRect(pos.x - 3, pos.y - 3, 6, 6);
 }
 
-- (void)setZoomFactor:(float)f
+- (void)setZoomFactor:(CGFloat)f
 {
     NSString *fname;
 
@@ -414,17 +451,33 @@
     if(!visible)
         return;
 
+    NSAssert (font != nil, @"Font object nil during drawing");
     style = [[NSMutableParagraphStyle alloc] init];
     [style setParagraphStyle:[NSParagraphStyle defaultParagraphStyle]];
     [style setAlignment: align];
     [style setParagraphSpacing: parspace];
     tempFont = [NSFont fontWithName:[font fontName] size:fsize*zmFactor];
-    strAttr = [[NSDictionary dictionaryWithObjectsAndKeys:
-      tempFont, NSFontAttributeName,
-      strokeColor, NSForegroundColorAttributeName,
-      fillColor, NSBackgroundColorAttributeName,
-      style, NSParagraphStyleAttributeName, nil] retain];
-
+    if (tempFont == nil)
+      {
+	NSLog(@"temp font obtained from %@ zoomFactor: %f is nil", font, zmFactor);
+	tempFont = font;
+      }
+    if (filled)
+      {
+	strAttr = [[NSDictionary dictionaryWithObjectsAndKeys:
+				   tempFont, NSFontAttributeName,
+				 strokeColor, NSForegroundColorAttributeName,
+				 fillColor, NSBackgroundColorAttributeName,
+				 style, NSParagraphStyleAttributeName, nil] retain];
+      }
+    else
+      {
+	strAttr = [[NSDictionary dictionaryWithObjectsAndKeys:
+				   tempFont, NSFontAttributeName,
+				 strokeColor, NSForegroundColorAttributeName,
+				 style, NSParagraphStyleAttributeName, nil] retain];
+      }
+    [style release];
     baselny = pos.y;
     bezp = [NSBezierPath bezierPath];
     [bezp setLineWidth:0];
@@ -442,10 +495,10 @@
             {
                 [[NSColor blackColor] set];
                 [bezp moveToPoint:NSMakePoint(pos.x, baselny)];
-                [bezp lineToPoint:NSMakePoint(pos.x + bounds.size.width, baselny)];
+                [bezp lineToPoint:NSMakePoint(pos.x + bounds.size.width*zmFactor, baselny)];
             }
 
-            baselny -= parspace;
+            baselny -= parspace*zmFactor;
         }
 
         if([editor isSelect])
@@ -455,6 +508,7 @@
             NSRectFill(selRect);
         }
     }
+  [strAttr release];
 } 
 
 @end
